<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2008 Christophe Monard <tof@cmonard.fr>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Librairie de fonction commune aux extensions
 *
 * @author	Christophe Monard   <tof@cmonard.fr>
 *
 * Appel direct:
 *  require_once(t3lib_extMgm::extPath('cmd_api').'class.tx_cmdapi.php');
 *	tx_cmdapi::*function_name*
 *
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *   54: class tx_cmdapi
 *   61:    function cleanTemplate($content)
 *   79:    function stripAccentsAndSpaces_doTransform($content, $spaceReplacer = '', $transform = '', $trueUTF8 = true)
 *  105:    function getCheckboxValues($table, $field, $row)
 *  124:    function getFieldsFromTable($table, $exclude = array('l10n_parent', 'l10n_diffsource'))
 *  144:	function sendMail($recipient, $param = array())
 *
 * TOTAL FUNCTIONS: 5
 *
 */

/**
 * Class permettant l'appel au fonctions
 *
 * @author	Christophe Monard <cmonard@free.fr>
 * @package	TYPO3
 * @subpackage	tx_cmdapi
 */
class tx_cmdapi {
	/**
	 * Fonction nettoyant le code des markers non utilisé
	 * 
	 * @param	string		$content: Contenu à traiter
	 * @return	Contenu généré
	 */
	function cleanTemplate($content) {
		// on nettoie le template des marquers inutilisé
		$pattern = "/###.+?###/is";
		$content = preg_replace($pattern, '', $content);

		// on renvoie le template préparé
		return $content;
	}

	/**
	 * Fonction enlevant accent et espace et faisant une transformation en minuscule, majuscule ou capitalise
	 * 
	 * @param	string		$content: Contenu à traiter
	 * @param	string		$spaceReplacer: le caractère de substitution pour les espaces
	 * @param	string		$transform: type de transformation ('', 'lower', 'upper', 'capitalise', 'first')
	 * @param	bool		$trueUTF8: set to false if db is different from utf-8 and forceCharset = utf-8
	 * @return	Contenu généré
	 */
	function stripAccentsAndSpaces_doTransform($content, $spaceReplacer = '', $transform = '', $trueUTF8 = true) {
		$accent = 'ÀÁÂÃÄÅàáâãäåÒÓÔÕÖØòóôõöøÈÉÊËèéêëÇçÌÍÎÏìíîïÙÚÛÜùúûüÿÑñ';
		if ($trueUTF8 && $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] && $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset'] == 'utf-8') {
			$content = utf8_decode($content);
			$accent = utf8_decode($accent);
		}
		$content = strtr($content, $accent, 'AAAAAAaaaaaaOOOOOOooooooEEEEeeeeCcIIIIiiiiUUUUuuuuyNn'); # do not handle utf-8 ??
		switch ($transform) {
			case 'lower': $content = strtolower($content); break;
			case 'upper': $content = strtolower($content); break;
			case 'capitalise': $content = ucwords($content); break;
			case 'first': $content = ucfirst($content); break;
			default: break;
		}
		$content = str_replace(' ', $spaceReplacer, $content);
		return $content;
	}

	/**
	 * Fonction générant la liste des éléments cochés d'un champ de type check sous forme de tableau associatif: uid => libellé
	 * 
	 * @param	string		$table: Nom de la table
	 * @param	string		$field: Nom du champ a traiter de la table
	 * @param	array		$row: Ligne courrante de la table
	 * @return	le tableau de valeurs pour la ligne courrante ou false en cas d'erreur
	 */
	function getCheckboxValues($table, $field, $row) {
		t3lib_div::loadTCA($table);
		$checkConfig = &$TCA[$table]['columns'][$field]['config'];
		if ($checkConfig['type'] && $checkConfig['type']=='check' && ($nbChecks = count($checkConfig['items']))>0) {
			$checks = array();
			for ($x=0; $x<$nbChecks; $x++)
				if ($row[$field]&pow(2,$x))
					$checks[$row['uid']][$x] = $GLOBALS['TSFE']->sL($checkConfig['items'][$x][0]);
		} else $checks = false;
		return $checks;
	}

	/**
	 * Fonction renvoyant un tableau des champs de la table données en parametre
	 * 
	 * @param	string		$table: Nom de la table pour laquelle on souhaite récupérer les champs
	 * @param	array		$exclude: liste des champs à exclure de la récupération TCA
	 * @return	tableau des champs de la table
	 */
	function getFieldsFromTable($table, $exclude = array('l10n_parent', 'l10n_diffsource')) {
		GLOBAL $TCA;
		t3lib_div::loadTCA($table);
		$columns = &$TCA[$table]['columns'];
		$fieldList = t3lib_div::trimExplode(',', $TCA[$table]['interface']['showRecordFieldList']); // liste des champs de la table

		$return = array();
		foreach ($fieldList as $field)
			if (!in_array($field, $exclude))
				$return[] = $field;
		return $return;
	}

	/**
	 * Fonction créant un mailling
	 * 
	 * @param	string		$recipient: Destinataire(s) du mail (eg@eg.com | example <eg@eg.com> | eg1@eg.com, eg2@eg.com | ...)
	 * @param	array		$param: Tableau des informations à utiliser (organisation, from_name, replyto_name, from_email, replyto_email, subject, bodyPlain, bodyHtml)
	 * @return	false en cas d'erreur
	 */
	function sendMail($recipient, $param = array()) {
		if ($recipient != '' && count($param > 0)) {
			$mailer=t3lib_div::makeInstance('t3lib_htmlmail');
			$mailer->start();
			$mailer->use8Bit();
			$mailer->organisation = $param['organisation']?$param['organisation']:'';
			$mailer->from_name =  $param['from_name']?$param['from_name']:'';
			$mailer->replyto_name = $param['replyto_name']?$param['replyto_name']:'';
			$mailer->from_email = $param['from_email']?$param['from_email']:'';
			$mailer->replyto_email = $param['replyto_email']?$param['replyto_email']:'';
			$mailer->subject = $param['subject']?$param['subject']:'';
			$mailer->recipient = $GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset']=='utf-8'?utf8_decode($recipient):$recipient;

			//pour des raison de fonctionnalité et pour être sur du résultat on fait un decode du html entity
			$bodyPLAIN = $param['bodyPlain']?html_entity_decode($param['bodyPlain'], ENT_COMPAT, ($GLOBALS['TYPO3_CONF_VARS']['BE']['forceCharset']=='utf-8'?'UTF-8':'')):'';
			$bodyHTML = $param['bodyHtml']?$param['bodyHtml']:'';
			if ($bodyPLAIN != '') $mailer->setPlain($bodyPLAIN);
			if ($bodyHTML != '') $mailer->setHtml($bodyHTML);

			//ajout des fichiers
			if ($param['files'] && is_array($param['files']))
				foreach ($param['files'] as $file)
					$mailer->addAttachment(PATH_site.$file);

			$mailer->setHeaders();
			$mailer->setContent();

			return $mailer->sendTheMail();
		} else return false;
	}

	/**
	 * Fonction créant un mailling
	 * 
	 * @param	array		$param: Tableau des instanciations à effectuer (lang, db, fe_user, be_user)
	 * @param	object		$connected: objet du fe_user connecté
	 * @param	object		$BE_USER: objet du be_user connecté
	 * @return	void
	 */
/*
	function initEID($param, &$connected = '', &$BE_USER = '') {
		if ($param['lang']) tslib_eidtools::initLanguage();
		if ($param['db']) tslib_eidtools::connectDB();
		if ($param['fe_user']) $connected = tslib_eidtools::initFeUser();
		if ($param['be_user']) {
			$BE_USER='';
			if ($_COOKIE['be_typo_user']) { // If the backend cookie is set, we proceed and checks if a backend user is logged in.
				require_once (PATH_t3lib.'class.t3lib_befunc.php');
				require_once (PATH_t3lib.'class.t3lib_userauthgroup.php');
				require_once (PATH_t3lib.'class.t3lib_beuserauth.php');
				require_once (PATH_t3lib.'class.t3lib_tsfebeuserauth.php');
			 
				// the value this->formfield_status is set to empty in order to disable login-attempts to the backend account through this script
				$BE_USER = t3lib_div::makeInstance('t3lib_tsfeBeUserAuth'); // New backend user object
				$BE_USER->OS = TYPO3_OS;
				$BE_USER->lockIP = $TYPO3_CONF_VARS['BE']['lockIP'];
				$BE_USER->start();  // Object is initialized
				$BE_USER->unpack_uc('');
				if ($BE_USER->user['uid'])  {
					$BE_USER->fetchGroupData();
					$TSFE->beUserLogin = 1;
				}
				// Now we need to do some additional checks for IP/SSL
				if (!$BE_USER->checkLockToIP() || !$BE_USER->checkBackendAccessSettingsFromInitPhp()) {
					// Unset the user initialization.
					$BE_USER='';
					$TSFE->beUserLogin=0;
				}
			}
		}
	}/**/
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cmd_api/class.tx_cmdapi.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/cmd_api/class.tx_cmdapi.php']);
}
?>